/*
 *   Oprogramowanie Robota
 *   Michal Slomkowski m.slomkowski@interia.pl, www.flylab.glt.pl
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "harddef.h"
#include <util/delay.h>

volatile static unsigned char analog_busy; 
volatile static int analog_result; 


// SIGNAL(SIG_ADC)
// {
// 	unsigned char adlow,adhigh;
// 	adlow=inp(ADCL); /* read low first !! */
// 	adhigh=inp(ADCH); /* read low first !! */
// 	analog_result=(adhigh<<8)|(adlow & 0xFF);
// 	analog_busy=0;
// }

int convertanalog(unsigned char channel) 
{
	analog_busy=1;
		unsigned char adlow,adhigh;
	/* enable analog to digital conversion in single run mode
	*  without noise canceler function. See datasheet of atmega8 page 195
	* We set ADPS2=1 and ADPS0,ADPS1=0 to have a clock division factor of 32.
	* This is needed to stay in the recommended range of 50-200kHz 
	* ADEN: Analog Digital Converter Enable
	* ADIE: ADC Interrupt Enable
	* ADIF: ADC Interrupt Flag
	* ADCSR: ADC Control and Status Register
	* ADPS2..ADPS0: ADC Prescaler Select Bits
	* REFS: Reference Selection Bits (page 203)
	*/

	// napiecie REF = 5V
	outp((1<<REFS0)|(channel & 0x07),ADMUX); 
	
        outp((1<<ADEN)|(1<<ADIE)|(1<<ADIF)|(1<<ADPS2),ADCSR);

	/*  start conversion */
	sbi(ADCSR,ADSC);

	//while(analog_busy); /* czekanie na przerwanie od przetwornika */
	_delay_ms(500);
	adlow=inp(ADCL); /* read low first !! */
	adhigh=inp(ADCH); /* read low first !! */
	analog_result=(adhigh<<8)|(adlow & 0xFF);
	analog_busy=0;
	
	return(analog_result);
}
