/*! \file main.c \brief Funkcja main() */
/*
 *   Oprogramowanie Robota
 *   Michal Slomkowski m.slomkowski@interia.pl, www.flylab.glt.pl
 */

#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "harddef.h"
#include <util/delay.h>

#include "rs.h"
#include "naped.h"
#include "analog.h"
#include "ds1621.h"
#include "misc.h"
#include "control.h"
#include "rc5.h"
#include "rc5_control.h"
#include "poszukiwanie_swiatla.h"

void start(void);

int main(void)
{
	uint8_t wybor = 1;

	//// inicjacje wszystkiego
	
	sei();				// przerwania
	rs_init(DEF_BAUD);		// port szeregowy
	naped_init();			// naped
	i2cInit();			// magistrala I2C
	rc5_init(0x0);			// pilot podczerwieni: jako TV1
	fdevopen(rs_put, rs_get);	// aktywacja standardowych funkcji wejscia/wyjscia
	buzzer_init();			// aktywacja wyjscia buzzera
	
	sbi(PORTD, PD3); // podciagniecie przyciskow do plusa
	sbi(PORTD, PD4); // jw.
	
	sbi(PORTB, PB1); // podciagniecie czujnika dotyku do plusa
	sbi(PORTB, PB2); // jw.
	
	UCSRB |= 1<<RXCIE; //wlaczenie przerwan od UARTu
	
	//// tekst powitalny przesylany przez RS232
	
	printf_P(PSTR("\r\n\nRobot wersja"SOFTWARE_VERSION"\r\nWcisnij h, aby uzyskac pomoc."));

	//// petla glowna

	while(1)
	{
		if((!(PINB & 1<<PB1)) | (!(PINB & 1<<PB2)))
		{
			// usuwanie drgan stykow
			_delay_ms(10);
			if((!(PINB & 1<<PB1)) | (!(PINB & 1<<PB2)))
			{
				buzzer_long();
				printf_P(PSTR("\r\nZderzenie!!\r"));
				naped_stop();
				_delay_ms(1000);
				naped_tyl();
				_delay_ms(2000);
				naped_stop();
				rc5.code = 0;
				_delay_ms(5000);
			}
		}
	}
}

//// sterowanie przez RC5, wywolywanie przez rc5.c
void rc5_sterowanie(void)
{
	//rc5.code = 0;
	
	asm("nop"); // nie wiem, czemu, bez tego nie chce dzialac, chyba blad kompilatora
	rc5.code++; // jw.
	rc5.code--; // jw.
	switch(rc5.code)
	{
		case RC5_PRZOD:
			naped_przod();
			break;
		case RC5_TYL:
			naped_tyl();
			break;
		case RC5_LEWO:
			naped_lewo();
			break;
		case RC5_PRAWO:
			naped_prawo();
			break;
		case RC5_STOP:
			naped_stop();
			break;
		case RC5_LED_ON:
			LED_ON();
			break;
		case RC5_LED_OFF:
			LED_OFF();
			break;
		case RC5_BUZZER:
			buzzer_long();
			break;
		case P_POSZ_SWIATLO_ON:
			if(poszukiwanie_on == 0)
			{
				poszukiwanie_on = TRUE;
				poszukiwanie_swiatla();
			}
			break;
		default:
			if(rc5.code == 0) break;
			printf_P(PSTR("\r\nKod rozkazu RC5: %d\r\n"), rc5.code);
	};
	rc5.code = 0;
}

SIGNAL(SIG_UART_RECV)
{
	asm("nop"); // nie wiem, czemu, bez tego nie chce dzialac, chyba blad kompilatora
	asm("nop"); // jw.

	switch(UDR)
	{
		case B_PRZOD:
			naped_przod();
			break;
		case B_TYL:
			naped_tyl();
			break;
		case B_LEWO:
			naped_lewo();
			break;
		case B_PRAWO:
			naped_prawo();
			break;
		case B_STOP:
			naped_stop();
			break;
		case B_LED_ON:
			LED_ON();
			break;
		case B_LED_OFF:
			LED_OFF();
			break;
		case B_SWIATLO:
			printf_P(PSTR("\r\nCzujnik lewy: %d\r\nCzujnik prawy: %d\r\n"),
				convertanalog(0), convertanalog(1));
			break;
		case B_TEMP:
			printf_P(PSTR("\r\nTemperatura: %d stopni C\r\n"), ds1621_temperatura());
			break;
		case B_POMOC:
			wyswietl_pomoc();
			break;
		case B_POSZ_SWIATLO_ON:
			if(poszukiwanie_on == 0)
			{
				poszukiwanie_on = TRUE;
				poszukiwanie_swiatla();
			}
			break;
		case B_BUZZER:
			buzzer_long();
			break;
		case B_RESET:
			RESET();	
	};
}

/* EOF */
