/*! \file poszukiwanie_swiatla.c
    \brief Algorytm odpowiedzialny za poszukiwanie zrodla swiatla i dazenie do niego */
/*
 *   Oprogramowanie Robota
 *   Michal Slomkowski m.slomkowski@interia.pl, www.flylab.glt.pl
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <math.h>

#include "poszukiwanie_swiatla.h"
#include "naped.h"
#include "analog.h"
#include "rc5.h"
#include "misc.h"
#include "harddef.h"

#include <util/delay.h>

#define MRUGNIJ() LED_ON();_delay_ms(254);LED_OFF();

uint8_t poszukiwanie_on;

void poszukiwanie_swiatla(void)
{
	int cz_l = 0, cz_p = 0;
	_delay_ms(254);
	_delay_ms(254);
	BOOL i = TRUE;
	
	while(poszukiwanie_on==1)
	{
		cz_l = convertanalog(0);
		cz_p = convertanalog(1);

		if(i)
		{
			if(cz_l > cz_p) naped_lewo();
			else if(cz_l < cz_p) naped_prawo();
			if(max_roznica(cz_l, cz_p)) i = FALSE;
		}
		else
		{
			MRUGNIJ();
			naped_stop();
			naped_przod();
			if(!(max_roznica(cz_l, cz_p))) i = TRUE;
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			_delay_ms(254);
			if(max(cz_l, cz_p) > max(convertanalog(0), convertanalog(1)))
			{
				naped_lewo();
				_delay_ms(254);
				_delay_ms(254);
				_delay_ms(254);
				naped_stop();
			}
		}
		if(cz_l >= 900)
		{
			MRUGNIJ();
			MRUGNIJ();
			MRUGNIJ();
			buzzer_long();
			printf_P(PSTR("\r\nDotarlem do swiatla!!!\r\n"));
		}
	}
}

static float max(float x, float y)
{
	if( x >= y) return x;
	else if( x < y) return y;
}

static BOOL max_roznica(int a, int b)
{
	int temp;
	if(a > b) temp = a - b;
	else if(a < b) temp = b - a;
	if(temp <= MAX_ROZNICA) return TRUE;
	else return FALSE;
}

/* EOF */
